
#ifndef _MAILLAGE_H
#define _MAILLAGE_H

#include <stdlib.h>

typedef struct
{
	float x, y, z; // Sa position 3D
}
Point;

typedef struct
{
	float r, v, b; // Rouge, Vert, bleu
}
Couleur;

/**
 *****************
 * Maillage simple
 *****************
 */
typedef struct
{
	// Nombre de points
	unsigned int NP;
	Point *points;
	Couleur *couls;
}
Maillage;


/**
 * Alloue un maillage de N triangles et le retourne
 */
static Maillage* allouerMaillage( unsigned int N )
{
	Maillage *m;
	m = (Maillage*) malloc(sizeof(Maillage));
	m->points = (Point*) calloc(N,sizeof(Point));
	m->couls = (Couleur*) calloc(N,sizeof(Couleur));
	m->NP = N;
	return m;
}

/**
 * Libere un maillage
 */
static void libererMaillage( Maillage *m )
{
	if( m )
	{
		free( m->points );
		free( m->couls );
		free( m );
		m = NULL;
	}
}





/**
 *****************
 * Maillage index
 *****************
 */
typedef struct
{
	int ind1, ind2, ind3; // Les indices des 3 points du triangle
}
TrianglePointIndex;

typedef struct
{
	// Nombre de points
	unsigned int NP;
	Point *points;
	Couleur *couls;
	// Pour les maillages  triangles indexs
	// Nombre de triangles
	unsigned int NT;
	TrianglePointIndex *indices;
	
}
MaillageIndexe;
 
static MaillageIndexe* allouerMaillageIndexe( unsigned int NP, unsigned int NT )
{
	MaillageIndexe *m;
	m = (MaillageIndexe*) malloc(sizeof(MaillageIndexe));
	m->points = (Point*) calloc(NP,sizeof(Point));
	m->couls = (Couleur*) calloc(NP,sizeof(Couleur));
	m->NP = NP;
	m->indices = (TrianglePointIndex*) calloc(NT,sizeof(TrianglePointIndex));
	m->NT = NT;
	return m;
}

/**
 * Libere un maillage
 */
static void libererMaillageIndexe( MaillageIndexe *m )
{
	if( m )
	{
		free( m->points );
		free( m->couls );
		free( m->indices );
		free( m );
		m = NULL;
	}
}

#endif
