#include <GL/glut.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>
#include "traqueboule.h"
#include "argumentParser.h"

#include "global.h"

#include "Maillage.h"
#include "dessiner.h"

// Utilis pour essayer diffrents types de rendu
// Utilis via le paramtre "-t" en ligne de commande
enum { TRIANGLE=0, SIMPLE=1, STRIP=2, GOURAUD=3, TETRA=5, SINANIM=4 };
unsigned int type = TRIANGLE;

unsigned int W_fen = 800;  // largeur fenetre
unsigned int H_fen = 800;  // hauteur fenetre

// espace couvert par la fonction
const float xmin = -1.0, xmax=1.0, ymin=-1.0, ymax = 1.0;
// echantillonnage du maillage
unsigned int NX = 10; // nombre de colonnes
unsigned int NY = 10; // nombre de lignes


// Le maillage de la surface associe  la fonction de Rosenbrock
Maillage *mRosen = NULL;
// Un ttrahdre pour les couleurs
MaillageIndexe *mTetra = NULL;
// Sinus anim
Maillage *mSin = NULL;

void animerSinus( Maillage *mSin );


void dessinerTriangle()
{
	// A COMPLETER
	// END COMPLETER
}


/**
 * Appel des diffrentes fonctions de dessin
*/
void dessiner( )
{
	switch( type )
	{
	case TRIANGLE:
		dessinerTriangle();
		break;
	case SIMPLE:
		dessinerMaillageMat( mRosen, NX, NY );
		break;
	case STRIP:
		dessinerMaillageMatStrip( mRosen, NX, NY );
		break;
	case GOURAUD:
		dessinerMaillageMatGouraud( mRosen, NX, NY );
		break;
	case SINANIM:
		dessinerMaillageMatGouraud( mSin, NX, NY );
		break;
	case TETRA:
		dessinerMaillage( mTetra, true );
		break;
	default:
		dessinerTriangle(); // Par dfaut
		break;
	}
}





/**
 * Remplit le maillage  mRosen
 */
void initRosen( )
{
    float zMax = 0;
    
    int NPoints;
    
    NPoints = (NX)*(NY);
    
    mRosen = allouerMaillage( NPoints );
    
    // Remplissage des donnes : positions et couleurs
    for( unsigned int i=0; i<NX; ++i )
    {
        for( unsigned int j=0; j<NY; ++j )
        {
		Point p;
		p.x = xmin +  i * (xmax-xmin)/(NX-1) ;
		p.y = ymin +  j * (ymax-ymin)/(NY-1) ;
		// fonction de Rosenbrock
		p.z = 100*(p.y-p.x*p.x)*(p.y-p.x*p.x) + (1-p.x)*(1-p.x);
		
		Couleur c;
		// couleur: nuance de jaune
		c.r = p.z;
		c.v = p.z;
		c.b = 0;

		mRosen->points[ind(i,j,NX)] = p;
		mRosen->couls[ind(i,j,NX)] = c;

		if(p.z>zMax)
		{
                	zMax=p.z; // pour ensuite tout ramener ultrieurement dans l'intervalle [0,1]
		}
        }
	}

    // mapping sur [0;1]
    for( unsigned int i=0; i<NX; ++i )
    {
        for( unsigned int j=0; j<NY; ++j )
        {
		mRosen->points[ind(i,j,NX)].z /= zMax;
		mRosen->couls[ind(i,j,NX)].r /= zMax;
		mRosen->couls[ind(i,j,NX)].v /= zMax;
		mRosen->couls[ind(i,j,NX)].b /= zMax;
        }
}

}

void initTetra()
{
	Point p0, p1, p2, p3;
	Couleur c0, c1, c2, c3;
	
	// 0 0 0 : noir
	p0.x = 0; p0.y = 0; p0.z = 0;	c0.r = 0; c0.v = 0; c0.b = 0;
	// 1 0 0 : rouge
	p1.x = 1; p1.y = 0; p1.z = 0;	c1.r = 1; c1.v = 0; c1.b = 0;
	// 0 1 0 : vert
	p2.x = 0; p2.y = 1; p2.z = 0;	c2.r = 0; c2.v = 1; c2.b = 0;
	// 0 0 1 : bleu
	p3.x = 0; p3.y = 0; p3.z = 1;	c3.r = 0; c3.v = 0; c3.b = 1;
	
	mTetra = allouerMaillageIndexe( 4, 4 );
	mTetra->points[0] = p0; mTetra->points[1] = p1; mTetra->points[2] = p2; mTetra->points[3] = p3;
	mTetra->couls[0] = c0; mTetra->couls[1] = c1; mTetra->couls[2] = c2; mTetra->couls[3] = c3;
	
	mTetra->indices[0].ind2 = 2; mTetra->indices[0].ind3 = 1; mTetra->indices[0].ind1 = 0;
	mTetra->indices[1].ind3 = 3; mTetra->indices[1].ind1 = 0; mTetra->indices[1].ind2 = 1; 
	mTetra->indices[2].ind1 = 0; mTetra->indices[2].ind2 = 3; mTetra->indices[2].ind3 = 2;
	mTetra->indices[3].ind2 = 1; mTetra->indices[3].ind3 = 2; mTetra->indices[3].ind1 = 3;
}

void initSinus()
{
    int NPoints;
    
    NPoints = (NX)*(NY);
    
    mSin = allouerMaillage( NPoints );
    
}



/**
 * Initiliasation et suppression des maillages
 */
void initDonnees()
{
	initRosen();
	initTetra();
	initSinus();
}

void finDonnees()
{
	libererMaillage( mRosen );
	libererMaillageIndexe( mTetra );
	libererMaillage( mSin );
}



/**
 * Animation du sinus
 */
static float t=0.0;
static float dt=0.01;
static float w=0.2;
static float weps=0.8;
void animerSinus( )
{
	// Remplit le maillage avec sin(t)
    // Remplissage des donnes : positions et couleurs
    for( unsigned int i=0; i<NX; ++i )
    {
        for( unsigned int j=0; j<NY; ++j )
        {
		Point p;
		p.x = xmin +  i * (xmax-xmin)/(NX-1) ;
		p.y = ymin +  j * (ymax-ymin)/(NY-1) ;
		// fonction sin
		float norm = sqrt(p.x*p.x+p.y*p.y);
		//p.z = sin(t-norm/w)*0.2*(1.0-norm);
		p.z = sin(t-norm/w)*0.2*exp(-norm/weps)/weps;
		
		Couleur c;
		c.r = fabs(p.z)/0.2;
		c.v = norm;
		c.b = fabs(sin(t+norm/w));

		mSin->points[ind(i,j,NX)] = p;
		mSin->couls[ind(i,j,NX)] = c;

        }
	}

	t += dt;
	display();
}






void display(void);
void reshape(int w, int h);
void keyboard(unsigned char key, int x, int y);

/**
 * Programme principal
 */
int main(int argc, char** argv)
{
	
    animal::parse("Maillages et illumination. Voici la liste des options: ")
    .option(&W_fen,'L',"Largeur","largeur de la fentre en pixels")
    .option(&H_fen,'H',"Hauteur","hauteur de la fentre en pixels")
    .option(&NX,'c',"colonnes","finesse de l'chantillonnage horizontal (nombre de colonnes")
    .option(&NY,'l',"lignes","finesse de l'chantillonnage vertical (nombre de lignes)")
    .option(&type,'t',"type","type de rendu")
    (argc,argv);

    glutInit(&argc, argv);

    // couches du framebuffer utilisees par l'application
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH );

    // position et taille de la fenetre
    glutInitWindowPosition(200, 100);
    glutInitWindowSize(W_fen,H_fen);
    glutCreateWindow(argv[0]);

     initDonnees( );
	

    // Initialisation du point de vue
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef(0,0,-4);
    tbInitTransform();     // initialisation du point de vue
    tbHelp();                      // affiche l'aide sur la traqueboule

    //
    // Active la lumire
    // Pour la partie
    // ECLAIRAGE
    /*     
    glEnable( GL_LIGHTING );
    glEnable( GL_LIGHT0 );
    glEnable(GL_COLOR_MATERIAL);
    int LightPos[4] = {0,0,3,1};
    int MatSpec [4] = {1,1,1,1};
    glLightiv(GL_LIGHT0,GL_POSITION,LightPos);
    glMaterialiv(GL_FRONT_AND_BACK,GL_SPECULAR,MatSpec);
    glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,10);
    */
    

    // cablage des callback
    glutReshapeFunc(reshape);
    glutKeyboardFunc(keyboard);
    glutDisplayFunc(display);
    glutMouseFunc(tbMouseFunc);    // traqueboule utilise la souris
    glutMotionFunc(tbMotionFunc);  // traqueboule utilise la souris
    glutIdleFunc( animerSinus );

    // lancement de la boucle principale
    glutMainLoop();
    
    finDonnees();
    
    return 0;  // instruction jamais excute
}

/**
 * Fonctions de gestion opengl  ne pas toucher
 */
// Actions d'affichage
// Ne pas changer
void display(void)
{
    // Details sur le mode de trac
    glEnable( GL_DEPTH_TEST );            // effectuer le test de profondeur
    //glEnable(GL_CULL_FACE);
    //glCullFace(GL_BACK);
    glPolygonMode(GL_FRONT,GL_FILL);
    glPolygonMode(GL_BACK,GL_LINE);
    glShadeModel(GL_FLAT);

    // Effacer tout
    glClearColor (0.0, 0.0, 0.0, 0.0);
    glClear( GL_COLOR_BUFFER_BIT  | GL_DEPTH_BUFFER_BIT); // la couleur et le z
    

    glLoadIdentity();  // repere camera

    tbVisuTransform(); // origine et orientation de la scene

    dessiner( );    

    glutSwapBuffers();
}
// pour changement de taille ou desiconification
void reshape(int w, int h)
{
    glViewport(0, 0, (GLsizei) w, (GLsizei) h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //glOrtho (-1.1, 1.1, -1.1,1.1, -1000.0, 1000.0);
    gluPerspective (50, (float)w/h, 1, 10);
    glMatrixMode(GL_MODELVIEW);
}

// prise en compte du clavier
void keyboard(unsigned char key, int x, int y)
{
    printf("key %d pressed at %d,%d\n",key,x,y);
    fflush(stdout);
    switch (key)
    {
    case 27:     // touche ESC
        exit(0);
    }
}

